/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: ImportSourceSelectionController.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.semantic.importexport;

import java.net.URISyntaxException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.File;
import java.io.Reader;
import java.net.URI;
import java.net.URL;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.io.SemanticUnmarshaller;
import it.tidalwave.semantic.io.SemanticUnmarshaller.Configuration;
import it.tidalwave.semantic.openrdf.OpenRDFSemanticFactory;
import it.tidalwave.semantic.persistence.NoSuchContextException;
import it.tidalwave.semantic.persistence.SemanticPersistence;
import org.jdesktop.beansbinding.AutoBinding;
import org.jdesktop.beansbinding.BeanProperty;
import org.jdesktop.beansbinding.BindingGroup;
import org.jdesktop.beansbinding.Bindings;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ImportSourceSelectionController.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
public class ImportSourceSelectionController extends AbstractSelectionController
  {
    private final static String CLASS = ImportSourceSelectionController.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public void doImport()
      throws NoSuchContextException, IOException, URISyntaxException
      {
        logger.fine("doImport()");

        ProgressInputStream inputStream = null;

        try
          {
            URI uri = null;
            final String mimeType = getUrlMIMEType();
            final SemanticPersistence semanticPersistence = SemanticPersistence.Locator.findSemanticPersistence();
            final OpenRDFSemanticFactory semanticFactory = OpenRDFSemanticFactory.Locator.findSemanticFactory();
            final Repository repository = semanticFactory.createDelegatingRepository(semanticPersistence.findRepositoryByName("DEFAULT"));
            final SemanticUnmarshaller unmarshaller = repository.getUnmarshaller();

            if (isFileSelected())
              {
                final File file = new File(getFile());
                uri = file.toURI();
                logger.finer(">>>> reading from file: %s - uri: %s - MIME type: %s", file.getAbsolutePath(), uri, mimeType);
                inputStream = new ProgressInputStream(file);
              }
            else if (isUrlSelected())
              {
                final URL url = new URL(getUrl());
                uri = url.toURI();
                logger.finer(">>>> reading from URL: %s - uri: %s - MIME type: %s", url, uri, mimeType);
                inputStream = new ProgressInputStream(url);
              }

            if (inputStream != null)
              {
                final BindingGroup bindings = new BindingGroup();
                bindings.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                                                               inputStream,
                                                               BeanProperty.create(inputStream.PROP_PROCESSED_BYTE_COUNT),
                                                               this,
                                                               BeanProperty.create(this.PROP_PROGRESSCOUNT)));
                bindings.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                                                               inputStream,
                                                               BeanProperty.create(inputStream.PROP_TOTAL_BYTE_COUNT),
                                                               this,
                                                               BeanProperty.create(this.PROP_SIZE)));
                bindings.bind();
                final Reader reader = new InputStreamReader(inputStream);
                final Configuration configuration = new Configuration();
                configuration.setBaseURI(uri);
                unmarshaller.unmarshal(reader, mimeType, configuration);
                bindings.unbind(); // FIXME: finally
              }
          }
        finally
          {
            if (inputStream != null)
              {
                try
                  {
                    inputStream.close();
                  }
                catch (IOException e)
                  {
                    logger.throwing(CLASS, "doImport()", e);
                  }
              }
          }
      }
  }
